.wikiwidget #toc {
    border:1px solid #aaaaaa;
    background-color:#f9f9f9;
    padding:5px;
}

h1, h2, h3, h4, h5, h6 {
	color: black;
	background: none;
	font-weight: normal;
	margin: 0;
	padding-top: .5em;
	padding-bottom: .17em;
	border-bottom: 1px solid #aaa;
}
h1 { font-size: 188%; }
h2 { font-size: 150%; }
h3, h4, h5, h6 {
	border-bottom: none;
}
h3 { font-size: 132%; }
h4 { font-size: 116%; }
h5 { font-size: 100%; }
h6 { font-size: 80%;  }

pre {
	background-color:#F9F9F9;
	border:1px dashed #2F6FAB;
	color:black;
	line-height:1.1em;
	padding:1em;
}
