from openerp.widgets.base import *
from openerp.widgets.interface import *

import form
import graph
import listgrid
import treegrid

import screen
import form_view

from js_i18n import js_i18n

# vim: ts=4 sts=4 sw=4 si et

