table.menubar {
    margin: 0;
    padding: 0;
    white-space: nowrap;
}

table.menubar td {
    margin: 0;
    padding: 0;
    border-right: 1px solid #333;
}

table.menubar a {
    display: block;
    text-align: center;
    background-color: #666;
    padding: 4px 10px;
    margin: 0 1px 0 0;
    color: #FFF;
    text-decoration: none
}

table.menubar a:hover {
    background-color: #505050;
    text-decoration: none;
}

table.menubar div.submenu {
    background-color: #666;
    border: 1px solid #333;
    position: absolute;
    left: 0px;
    visibility: hidden;
    margin-left:  -1px;
    z-index: 3
}

table.menubar div.submenu a {
    display: block;
    text-align: left;
    text-decoration: none;
    color: #FFF;
    margin: 0px;
}

table.menubar div.submenu a:hover {
    background-color: #505050;
    text-decoration: none;
}

div.contextmenu {
    background-color: #444;
    border: 1px solid #000;
    white-space: nowrap;
    position: absolute;
    padding: 2px;
    z-index: 100;
}

div.contextmenu a,
div.contextmenu span {
    display: block;
    text-align: left;
    text-decoration: none;
    white-space: nowrap;
    padding: 2px;
    color: #FFF;
    cursor: pointer;
}

div.contextmenu a:hover,
div.contextmenu span:hover {
    color: #FFF;
    background-color: #505050;
    text-decoration: none;
}

div.contextmenu a.disabled,
div.contextmenu span.disabled {
    color: #C0C0C0;
}

/**
 * vim: sts=4 st=4 et 
 *
 */
