.calMonth,
.calWeek {
    background-color: #E0E0E0;
    padding: 8px;
    padding-top: 4px;
    position: relative;
}

.calDayHeader {
    height: 21px;

    text-align: center;
    vertical-align: middle;
    white-space: nowrap;

    font-weight: bold;
    font-size: 12px;
}

.calWeek a {
    text-decoration: none;
    color: black;
}

#calHeaderSect,
#calAllDaySect {
    height: 21px;
    position: relative;
    overflow: hidden;
}

#calAllDaySect {
    margin-bottom: 6px;
    border: 1px inset white;
    background-color: #ffffff;
}

#calAllDaySect .droppable {
    background-color: #BFE4FF;
}

.calWeek #calHeaderSect,
.calWeek #calAllDaySect {
    margin-left: 70px;
    margin-right: 16px;
}

.calMonth #calHeaderSect {
    margin-left: 35px;
}

#calBodySect {
    position: relative;
    border: 1px inset white;
    background-color: #ffffff;
}

.calWeek #calBodySect {
    height: 480px;
    overflow: auto;
    overflow-y: scroll;
    overflow-x: hidden;
}

.calWeek .calVRule {
    position: absolute;
    top: 0pt;
    width: 1px;

    border-left: 2px solid #C0C0C0;
}

.calWeek .calHRule {
    height: 19px;
    background-color: #FFFFFF;
    border-top: 1px solid #F0F0F0;
}

.calWeek .calHRule.even {
    height: 18px;
    border-top: 2px solid #F0F0F0;
}


.calTimeCol {
    width: 70px;
    font-size: 10px;
    font-weight: bold;
    white-space: nowrap;
}

.calTimeCol div {
    height: 38px;
    border-top: 2px solid #E0E0E0;
    background-color: #F3F3F3;
    padding-right: 2px;
    text-align: right;
}

.calEvent,
.calEventNew {
    position: absolute;
    z-index: 100;

    padding-left: 2px;

    font-size: 10px;
    white-space: nowrap;
    overflow: hidden;

    cursor: pointer;
}

.calWeek #calBodySect .calEvent,
.calWeek #calBodySect .calEventNew {
    padding: 0px;
    margin: 0px;

    font-size: 9px;

    border: 2px solid black;
    cursor: default;
}

.calEventTitle {
    color: white;
    font-weight: bold;
    padding-bottom: 2px;
    white-space: nowrap;
    cursor: move;
}

.calEventDesc {
    color: black;
    padding: 2px;
    vertical-align: top;
    font-size: 10px;
}

.calEventGrip {
    position: absolute;
    padding: 0px;

    left: 0;
    bottom: 1px;

    width: 100%;
    height: 5px;

    background: url("/static/tinycalendar/images/calGrip.gif") no-repeat scroll center bottom;
    cursor: s-resize;
}

#calLoading {
    color: maroon;
    font-weight: bold;
    padding-left: 20px;
    background: url("/static/tinycalendar/images/calLoading.gif") no-repeat left;
    display: none;
}

/**************************************/

.calMonthWeek {
    position: relative;
    width: 100%;
    height: 121px;
}

.calMonth .calTimeCol {
    width: 35px;

    font-size: 18px;
    font-weight: bold;
}

.calMonth .calTimeCol div {
    background-color: #E0E0E0;
    border-top: 1px solid #C0C0C0;
}

.calMonth .calGrid {
    position: relative;
    overflow: hidden;
}

.calMonthDay {
    position: absolute;
    vertical-align: top;

    background-color: white;
    border-left: 1px solid #C0C0C0;

    font-size: 10px;
}

.calMonthDay.dayOff {
    background-color: #E9E9E9;
}

.calMonthDay.dayThis {
    background-color: #FFFFDF;
}

.calMonthDayTitle {
    height: 16px;
    font-size: 10px;
    font-weight: bold;
    text-align: right;
    padding-right: 4px;

    border-top: 1px solid #C0C0C0;
    background-color: #D9D9D9;
}

.calMonthDay.dayThis .calMonthDayTitle {
    background-color: #E6E6A1;
}

#calAllDaySect .droppable,
.calMonthDay.droppable {
    background-color: #BFE4FF;
}

.calMonthDay.droppable .calMonthDayTitle{
    background-color: #80C9FF;
}

.calMonthDay .calEventInfo {
    position: absolute;
    bottom: 1px;
}

.calMonthDay .calEventInfo,
.calMonthDay .calEventInfo a {
    color: navy;
    padding-left: 2px;
}

.calMonthDay .calEventInfo a {
    text-decoration: none;
}

/**
 * Info Dialog
 */

#calInfoLayer {
    display: none;

    position: absolute;
    top: 0;
    left: 0;
    z-index: 199;

    width: 100%;
    height: 100%;

    background-color: gray;
    opacity: 0.3;
}

#calInfoBox {
    position: absolute;
    z-index: 200;

    background-color: white;
    border: 2px solid #696969;
}

#calInfoBox .calInfoTitle {
    padding: 2px;
    background-color: #696969;
    color: white;
    font-weight: bold;
}

#calInfoBox .calInfoDesc {
    padding: 2px;
}

#calInfoBox .calInfoButtons {
    position: absolute;
    bottom: 0pt;
    padding: 2px;
    border-top: 2px solid #696969;
}

/**
 * Mini Calendar
 */

#MiniCalendar {
    background-color: #E0E0E0;
    padding: 8px;
}

.calMini {
    font-size: 10px;
    text-align: center;
}

.calMiniTitles td {
    width: 14%;
}

.calMiniDays {
    background-color: white;
}

.calMiniDays .dayOff {
    background-color: #E9E9E9;
}

.calMiniDays .dayThis {
    background-color: #FFFFDF;
}

.calMini .weekSelected td,
.calMini td.daySelected {
    background-color: navy;
}

.calMini .weekSelected td a,
.calMini td.daySelected a {
    color: white;
}

.calMini .button {
    cursor: pointer;
}

/**
 * Group Box
 */

#MiniCalendar {
    margin-bottom: 5px;
}

#calSearchOptions,
#calGroups {
    margin: 0pt;
    margin-bottom: 5px;
    padding: 8px;
    background-color: #E0E0E0;
}

#calSearchOptions {
    padding-left: 12px;
}

#calGroups dt {
    font-weight: bold;
    margin-bottom: 2px;
}

#calGroups dd {
    margin-left: 4px;
}

.calMini a,
.calMonth a,
.calWeek a {
    text-decoration: none;
    color: black;
}

/**
 * vim: sts=4 st=4 et
 *
 */
