.pager {
    font-size: 11px;
    font-weight: bold;
    color: #999;
    vertical-align: middle;
}

.pager a {
    color: #333;
}

.pager a:hover {
    text-decoration: none;
}

/**
 * vim: sts=4 st=4 et 
 *
 */
