
.stateName {
    position: absolute;
    font-size: 12px;
    width: 95%;
    left: 0;
    top: 20px;
    z-index: 1000;
    overflow: hidden;
    
}

#calInfoLayer {
    display: none;
    position: absolute;
    top: 0;
    left: 0;
    z-index: 199;
    width: 100%;
    height: 100%;
    background-color: gray;
    opacity: 0.3;
}

#calInfoBox {
    position: absolute;
    z-index: 10001;
    background-color: white;
    border: 2px solid #696969;
}

.calMini .button {
    cursor: pointer;
}

#calInfoBox .calInfoTitle {
    padding: 2px;
    background-color: #696969;
    color: white;
    font-weight: bold;
}

#calInfoBox .calInfoDesc {
    padding: 2px;
}

#calInfoBox .calInfoButtons {
    position: absolute;
    bottom: 0pt;
    padding: 2px;
    border-top: 2px solid #696969;
}