#process_canvas {
    position: relative;
    margin: auto;
    outline: none;
}

.process-notes {
    font-size: 11px;
    max-width: 250px;
    max-height: 250px;
    text-align: left;
    position: absolute;
    left: 10px;
    z-index: 1000;
}

.process-notes dt {
    font-weight: bold;
}

.process-notes dd {
    margin-left: 15px;
    margin-bottom: 4px;
}

.node {
    width: 150px;
    height: 100px;
    text-align: left;
}

.node-title {
    padding-left: 45px;
    padding-top: 1px;
    height: 15px;
    font-size: 10px;
    font-weight: bold;
    color: white;
    overflow: hidden;
}

.node-title a:hover,
.node-title a {
    color: white;
    text-decoration: underline;
}

.node-text {
    font-size: 11px;
    padding: 2px;
    padding-left: 20px;
    height: 60px;
    overflow: hidden;
}

.node-bottom {
    padding-left: 20px;
    height: 18px;
}

.node-menu {
    padding-right: 6px;
    width: 100%;
}

.node-bottom table {
    border-collapse: collapse;
}

.node-bottom img {
    cursor: pointer;
    margin: 0px 2px;
}

